/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.gossip;

import java.io.BufferedReader;
import java.io.File;
import java.io.PrintStream;
import java.util.Collection;
import java.util.Date;
import java.net.InetSocketAddress;

import casa.dodwan.config.DodwanEnvironment;
import casa.dodwan.docware.Descriptor;
import casa.dodwan.docware.Payload;
import casa.dodwan.message.Message;
import casa.dodwan.message.MessageBufferizer;
import casa.dodwan.util.ArgumentNotFoundException;
import casa.dodwan.util.ArgumentParsing;
import casa.dodwan.util.ArgumentParsingException;
import casa.dodwan.util.Command;
import casa.dodwan.util.Console;
import casa.dodwan.util.DataFormatting;
import casa.dodwan.util.Environment;
import casa.dodwan.util.Host;

public class GossipingServiceConsole extends Console {
    
    public GossipingService gossipingService_;
    
    // ------------------------------------------------------------
    public GossipingServiceConsole(GossipingService gossipingService) {
	
	gossipingService_ = gossipingService;
	prompt_ = "gossiping% ";
    }
    
    // ============================================================
    // MAIN COMMANDS
    // ============================================================
    
    // ------------------------------------------------------------
    public GossipingServiceConsole(String prompt, GossipingService gossipingService) {
	
	gossipingService_ = gossipingService;
	prompt_ = prompt;
    }
    
    // // ------------------------------------------------------------
    // public void gossipingAdvertise(String[] arguments, PrintStream out) {
	
    // 	if (arguments.length != 1) {
    // 	    help("  ", out);
    // 	    return;
    // 	}
	
    // 	try {
    // 	    gossipingService_.advertiser.proceed();
    // 	}
    // 	catch (Exception e) {
    // 	    out.println(e);
    // 	}
    // }
 
    
    // ------------------------------------------------------------
    public void gossipingDisable(String[] arguments, PrintStream out) {
	
	if (arguments.length < 1) {
	    help("  ", out);
	    return;
	}
	
	if (arguments.length == 1) {
	    gossipingService_.disable();
	    return;
	}
	
	for (int i = 1; i < arguments.length; i++) {
	    //  			if (arguments[i].startsWith("p"))
	    //  				gossipingService_.environment.disable();
	    
	    // if (arguments[i].startsWith("a"))
	    // 	gossipingService_.advertiser.disable();
	    
	    // if (arguments[i].startsWith("f"))
	    // 	gossipingService_.setFiltering(false);
	}
    }
    
    // ------------------------------------------------------------
    public void gossipingEnable(String[] arguments, PrintStream out) {
	
	if (arguments.length < 1) {
	    help("  ", out);
	    return;
	}
	
	if (arguments.length == 1) {
	    gossipingService_.enable();
	    return;
	}
	
	for (int i = 1; i < arguments.length; i++) {
	    //  			if (arguments[i].startsWith("p"))
	    //  				gossipingService_.environment.enable();
	    
	    // if (arguments[i].startsWith("a"))
	    // 	gossipingService_.advertiser.enable();
	    
	    // if (arguments[i].startsWith("f"))
	    // 	gossipingService_.setFiltering(true);
	}
	
    }
    
    // ------------------------------------------------------------
    private String humanReadable(long value) {

	if (value >= 1048576)
	    return String.valueOf(value / 1048576) + "MB";
	if (value >= 1024)
	    return String.valueOf(value / 1024) + "kB";
	return String.valueOf(value) + "B";
    }
    
    // ------------------------------------------------------------
    public void gossipingStatus(String[] arguments, PrintStream out) {

	out.println("enabled: " + gossipingService_.isEnabled());
	out.println("suspended: " + gossipingService_.isSuspended());
	out.println();
	
	Environment env = gossipingService_.environment;
	Host localHost = env.getLocalHost();

	Collection<Host> peers = env.getPeers();
	long now = System.currentTimeMillis();

	out.println(DataFormatting.toString("Id", 10) + "\t"
		    + "pid\toffer\tdup\texp\tavail\telapsed\n");
	out.println(DataFormatting.toString(localHost.id, 10) + "\t" 
		    + localHost.pid + "\t"
		    + "--\t"
		    + "--\t"
		    + "--\t"
		    + "--\t"
		    + ((now - localHost.lastAnnounce) / 1000) + "\t"
		    + "(local host)");
	
	for (Host peer: peers) {
	    out.println(DataFormatting.toString(peer.id, 10) + "\t"
			+ peer.pid + "\t"
			+ peer.couldUse.size() + "\t"
			+ peer.duplicates.size() + "\t"
			+ (peer.expected == null?"0":"1") + "\t"
			+ peer.canProvide.size() + "\t"
			+ ((now - peer.lastAnnounce) / 1000));
	    // + "\t" + env.getAddress(peer.id));
	}
    }
    
    // ------------------------------------------------------------
    public void gossipingResume(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("  ", out);
	    return;
	}
	
	gossipingService_.resume();
    }

    // ------------------------------------------------------------
    public void gossipingSendTime(String[] arguments, PrintStream out) {

	if (arguments.length == 1)
	    gossipingService_.sendTime(null);
	else
	    for (int i=1; i<arguments.length; i++)
		gossipingService_.sendTime(arguments[i]);
    }

    // ------------------------------------------------------------
    public void gossipingSuspend(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("  ", out);
	    return;
	}
	
	gossipingService_.suspend();
    }
    
    // ------------------------------------------------------------
    public void gossipingVerbosity(String[] arguments, PrintStream out) {
	
	if (arguments.length != 2) {
	    help("  ", out);
	    return;
	}
	
	int level = -1;
	try {
	    level = Integer.parseInt(arguments[1]);
	}
	catch (Exception e) {
	    out.println("Could not parse verbosity level");
	}
	
	if (level > 0)
	    gossipingService_.verbosity.enable();
	else 
	    gossipingService_.verbosity.disable();
	;
    }
    
    // ------------------------------------------------------------
    @Override
	public void help(String offset, PrintStream out) {
	out.println(offset
		    + "[ena]ble / [d]isable\n"
		    + offset
		    + "[su]spend / [res]ume\n"
		    // + offset
		    // + "[a]dd <peer> <address>\n"
		    // + offset
		    // + "[l]ist\n"
		    // + offset
		    // + "[rem]ove <peer>\n"
		    + offset
		    + "[sen]dTime <host>*\n"
		    + offset
		    + "[st]atus\n"
		    + offset
		    + "[v]erbosity <level>");
		    // + offset + "[fi]lter <-desc pattern>\n"
		    // + offset + "[p]rofile <-desc pattern>\n"
		    // + offset + "[a]dvertise");
    }
    
    // ------------------------------------------------------------
    @Override
	public void processCommand(String[] arguments, BufferedReader reader,
				   PrintStream out) {
	
	if (arguments.length < 1) {
	    help("  ", out);
	    return;
	}
	
	// if (arguments[0].startsWith("a")) {
	//     gossipingAdvertise(arguments, out);
	//     return;
	// }
	
	if (arguments[0].startsWith("di")) {
	    gossipingDisable(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("ena")) {
	    gossipingEnable(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("h")) {
	    out.println("Gossiping Service: Command Set\n");
	    help("  ", out);
	    return;
	}
	
	if (arguments[0].startsWith("res")) {
	    gossipingResume(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("sen")) {
	    gossipingSendTime(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("st")) {
	    gossipingStatus(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("su")) {
	    gossipingSuspend(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("v")) {
	    gossipingVerbosity(arguments, out);
	    return;
	}
	
	out.println("Unknown command");
	help("  ", out);
    }
    
    // ============================================================
    // COMMAND PARSING
    // ============================================================
    
    // ------------------------------------------------------------
    public void resume(String[] arguments, PrintStream out) {
	
	if (arguments.length != 0) {
	    help("  ", out);
	    return;
	}
	
	gossipingService_.resume();
    }
    
    // ------------------------------------------------------------
    public void source(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("  ", out);
	    return;
	}
	
	parseCommandFile(arguments[0], out);
    }
    
    // ------------------------------------------------------------
    public void suspend(String[] arguments, PrintStream out) {
	
	if (arguments.length != 0) {
	    help("  ", out);
	    return;
	}
	
	gossipingService_.suspend();
    }
    
}
