/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.hub;

import java.util.Collection;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;

public class FullConnectivityEnvironment
    implements Environment {
    
    private Map<String, Boolean> nodes_ = new HashMap<String, Boolean>();

    // ------------------------------------------------------------
    public void addNode(String id, boolean online) {

	nodes_.put(id, new Boolean(online));
    }
    
    // ------------------------------------------------------------
    public void deleteNode(String id) {

	nodes_.remove(id);
    }

     // ------------------------------------------------------------
    public void setNode(String id, boolean online) {

	nodes_.put(id, new Boolean(online));
    }
    
     // ------------------------------------------------------------
    public void setNodeStatus(String id, String status) {}
    
     // ------------------------------------------------------------
    public void setConnectionStatus(String id1, String id2,
				    boolean connected) {}
    
     // ------------------------------------------------------------
    public boolean isOnline(String id) {

	Boolean isOnline = nodes_.get(id);
	if (isOnline == null) return false;

	return isOnline.booleanValue();
    }
    
   // ------------------------------------------------------------
    public Set<String> getNodes() {

	return nodes_.keySet();
    }

    // ------------------------------------------------------------
    public boolean isNode(String id) {

	return nodes_.containsKey(id);
    }

    // ------------------------------------------------------------
    public boolean arePeers(String id1, String id2) {

	return (isOnline(id1) && isOnline(id2));
    }

    // ------------------------------------------------------------
    public Collection<String> getPeers(String id) {

	Set<String> result = new HashSet<String>();

	Boolean online = nodes_.get(id);
	if ((online == null) || (! online.booleanValue()))
	    return result;
	    
	for (String k: nodes_.keySet())
	    result.add(k);

	result.remove(id);

	return result;
    }
 
}
