/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.message;

import casa.dodwan.docware.Descriptor;
import casa.dodwan.docware.DescriptorBufferizer;
// import casa.dodwan.docware.DescriptorZipBufferizer;
import casa.dodwan.util.BufferAccessException;
import casa.dodwan.util.Bufferizer;
import casa.util.DataBufferizer;
import casa.util.Logger;
import casa.dodwan.util.Verbosity;

public class MessageBufferizer extends Bufferizer<Message> {

    private Bufferizer<Descriptor> descriptorBufferizer_ = 
	// new DescriptorZipBufferizer();
	new DescriptorBufferizer();

    public Verbosity verbosity = new Verbosity();
    
    // ------------------------------------------------------------
    public Message fromBuffer(byte[] buffer, int offset, int length)
	throws BufferAccessException {

	 // Logger.log("mbuf", "fromBuffer(" + buffer
	 // 	   + ", " + offset
	 // 	   + ", " + length
	 // 	   + ")");

	if (verbosity.isEnabled()) {
	    System.out.println("MessageBufferizer.fromBuffer(" + buffer + ", "
			       + offset + ", " + length + ")");
	}
	
	Message message = new Message();
	
	try {
	    // Reading descriptor length
	    int hLength = DataBufferizer.readInt(buffer, offset);
	    offset += 4;

	    // Reading descriptor
	    Descriptor desc =
		descriptorBufferizer_.fromBuffer(buffer, offset, hLength);
	    // // Decompressing data
	    message.setDescriptor(desc);
	    offset += hLength;

	    // Reading payload
	    int pLength = length - 4 - hLength;
//  	    if (pLength > 0) {
//	    System.out.println("pLength=" + pLength);
		byte[] payload = new byte[pLength];
		System.arraycopy(buffer, offset, payload, 0,
				 pLength);
		message.setPayload(payload);
//  	    }

// 	    System.out.println("Message.fromBuffer(): hlength="
// 			       + hLength + ", plength="
// 			       + pLength);
 	}
	catch (Exception e) {
	    Logger.log("mbuf", "Failed to decode message");
	    throw new BufferAccessException("MessageBufferizer.fromBuffer(...): " + e);
	}
	
	return message;
    }
    
    // ------------------------------------------------------------
    public byte[] toBuffer(Message message)
	throws BufferAccessException {

	 // Logger.log("mbuf", "toBuffer(" + message + ")");

// 	if (verbosity.isEnabled()) {
// 	    System.out.println("MessageBufferizer.toBuffer(" + message + ", "
// 			       + buffer + ", " + offset + ")");
// 	}

	byte[] result = null;

	try {
	    // Preparing descriptor
	    Descriptor desc = message.getDescriptor();
	    byte[] hbuf = descriptorBufferizer_.toBuffer(desc);
	    int hlength = hbuf.length;

	    byte[] pbuf = message.getPayload();
	    int plength = (pbuf == null?0:pbuf.length);

	    result = new byte[hlength + 4 + plength];

	    // Writing descriptor
	    System.arraycopy(hbuf, 0, result, 4, hlength);

	    // Writing descriptor length
	    DataBufferizer.writeInt(hlength, result, 0);

	    // Writing payload
	    int offset = 4 + hlength;
	    if (pbuf != null) 
		System.arraycopy(pbuf, 0, result, offset, plength);
	}
	catch (Exception e) {
	    throw new BufferAccessException("MessageBufferizer.toBuffer(...): "
					    + e);
	}
	
	return result;
    }

    // ------------------------------------------------------------
    public int toBuffer(Message message, byte[] buffer, int offset)
	throws BufferAccessException {

	 // Logger.log("mbuf", "toBuffer(" + buffer
	 // 	   + ", " + offset
	 // 	   + ")");

// 	if (verbosity.isEnabled()) {
// 	    System.out.println("MessageBufferizer.toBuffer(" + message + ", "
// 			       + buffer + ", " + offset + ")");
// 	}
	
	int length = 0;
	try {
	    // Writing descriptor
	    Descriptor desc = message.getDescriptor();
	    int hLength = descriptorBufferizer_.toBuffer(desc, buffer, offset + 4);

	    // Writing descriptor length
	    DataBufferizer.writeInt(hLength, buffer, offset);
	    offset += 4 + hLength;
	    length += 4 + hLength;
	    
	    // Writing payload
	    byte[] payload = message.getPayload();
	    if (payload != null) {
		System.arraycopy(payload, 0, buffer, offset,
				 payload.length);
		length += payload.length;
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	    throw new BufferAccessException("MessageBufferizer.toBuffer(...): "
					    + e);
	}
	
	return length;
    }
    
}
