/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.message;

import casa.dodwan.docware.Descriptor;
import casa.util.Processor;
import casa.dodwan.util.Sink;
import casa.dodwan.util.Verbosity;

public class MessageReceiver implements Sink<Descriptor> {
    
    private Descriptor pattern_;
    private Processor<Descriptor> processor_;

    public Verbosity verbosity = new Verbosity();
    
    // ------------------------------------------------------------
    public MessageReceiver(Descriptor pattern, 
			   Processor<Descriptor> processor) 
	throws Exception {

	pattern_ = pattern;
	processor_ = processor;
    }
    
    // ------------------------------------------------------------
    public Descriptor getPattern() {
	return pattern_;
    }
    
    // ------------------------------------------------------------
    public Processor<Descriptor> getProcessor() {
	return processor_;
    }
    
    // ------------------------------------------------------------
    public void setProcessor(Processor<Descriptor> processor) {
	processor_ = processor;
    }
    
    // ------------------------------------------------------------
    public String toString() {

	return "MessageReceiver(pattern="
	    + pattern_ + ", processor="
	    + processor_ + ")";
    }

    // ------------------------------------------------------------
    public void write(Descriptor desc) throws Exception {
	
	if (verbosity.isEnabled())
	    System.out.println("MessageReceiver.write('"
			       + desc.getDocumentId() + "')");
	// System.out.println("pattern=" + pattern_);
	// System.out.println("desc   =" + desc);
	
	if ((pattern_ == null) || desc.matches(pattern_)) {
	    // if (verbosity.isEnabled())
		// System.out.println("  Accepted matching document");

	    if (processor_ != null) {
		processor_.process(desc);
	    }
	}
    }
    
}
