/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/





package casa.dodwan.net;

import java.net.NetworkInterface;
import java.net.InetAddress;
import java.net.Inet6Address;
import java.net.Inet4Address;
import java.util.Enumeration;

public class browseInterfaces {

    private static char[] digits = 
    {'0','1','2','3','4','5','6','7','8','9','A','B','C','D',
'E','F'};

    //------------------------------------------------------------
    public static String hexa(byte v) {
        
        int low  = v & 0x0F;
        int high = (v & 0xF0) >> 4;
        return String.valueOf(digits[high]) 
            + String.valueOf(digits[low]);
        
    }

     //------------------------------------------------------------
   public static String toString(byte[] address) {

	String result = null;
	for (byte b: address)
	    if (result == null) 
		result = hexa(b);
	    else
		result = result + ":" + hexa(b);
	return result;
    }

    //------------------------------------------------------------
    public static void browse() throws Exception {

	for (Enumeration e = NetworkInterface.getNetworkInterfaces() ; e.hasMoreElements() ;) {
	    NetworkInterface iface = (NetworkInterface)e.nextElement();
	    byte[] hwadd = iface.getHardwareAddress();
	    System.out.println("\nName : " + iface.getDisplayName()
			       + (iface.isUp()?" (UP)":" (DOWN)")
			       + (iface.supportsMulticast()?" (MCAST)":""));
	    if (hwadd != null) 
		System.out.println("MAC  : " + toString(hwadd));
	    for (Enumeration<InetAddress> a = iface.getInetAddresses(); a.hasMoreElements();) {
		InetAddress add = a.nextElement();
		System.out.println((add instanceof Inet6Address?"Inet6: ":"Inet4: ")
				   + add);
	    }

//  	    System.out.println("Display name: " + iface.getDisplayName());
//  	    for (Enumeration f = iface.getInetAddresses() ; f.hasMoreElements() ;) {
//  		Object addr = f.nextElement();
//  		    System.out.println("  addr: " + addr);
//  	    }
	}
    }

    // ------------------------------------------------------------
    public static InetAddress getInet4Address() {

	try {
	    for (Enumeration e =
		     NetworkInterface.getNetworkInterfaces() ; e.hasMoreElements() ;) {
		NetworkInterface iface = (NetworkInterface)e.nextElement();
		// System.out.println("\nName : " + iface.getDisplayName()
		// 		   + (iface.isUp()?" (UP)":" (DOWN)")
		// 		   + (iface.supportsMulticast()?" (MCAST)":""));
		if (iface.isUp()) {
		    for (Enumeration<InetAddress> a =
			     iface.getInetAddresses(); a.hasMoreElements();) {
			InetAddress add = a.nextElement();
			if ((add instanceof Inet4Address)    // IPv4 address
			    && (! add.isAnyLocalAddress())   // Not wildcard
			    && (! add.isLoopbackAddress())   // Not loopback
			    && (! add.isLinkLocalAddress())) // Not link local
			    return add;
		    }
		}
	    }
	}
	catch (Exception e) {
	    System.out.println(e);
	}
	
	return null;
    }
    

    // ------------------------------------------------------------
    public static void main(String[] args) throws Exception {

	if (args.length == 0) {
	    browse();
	    return;
	}

	if (args[0].equals("-h")) {
	    System.out.println("browseInterfaces [-l|...]");
	    return;
	}

	if (args[0].equals("-l")) {
	    while (true) {
		browse();
		System.out.println("---------------------------------------");
		Thread.sleep(2000);
	    }
	}
    }
}
