/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.pubsub;

import java.net.InetSocketAddress;
import java.net.Socket;
import java.io.OutputStream;

import casa.dodwan.message.Message;
import casa.dodwan.message.MessageBufferizer;
import casa.dodwan.docware.Descriptor;
import casa.dodwan.net.InetSocket;
import casa.dodwan.net.Stream_IO;
import casa.util.Processor;
import casa.dodwan.pubsub.PubSubService;

public class MessageForwarder implements Processor<Descriptor> {

    private InetSocketAddress addr_;
    private PubSubService pubsub_;
    private MessageBufferizer bufferizer_ = new MessageBufferizer();

    public MessageForwarder(InetSocketAddress addr, PubSubService pubsub) {

	// System.out.println("Creating forwarder to " + addr);
	addr_ = addr;
	pubsub_ = pubsub;
    }
    
    // ------------------------------------------------------------
    public void process(Descriptor desc) throws Exception {

	System.out.println("Forwarding msg to " + addr_);

	try {
	    Socket socket = new Socket(addr_.getAddress(), addr_.getPort());
	    OutputStream out = socket.getOutputStream();
	    String key = desc.getDocumentId();
	    byte[] payload = pubsub_.getAsBytes(key);
	    Message msg = new Message(desc, payload);
	    
	    byte[] buffer = bufferizer_.toBuffer(msg);
	    Stream_IO.sendBundle(out, buffer);
	    out.flush();
	    socket.close();
	}
	catch (Exception e) {
	    System.out.println("MessageForwarder: failed to send msg to " + addr_);
	}
    }
	

    // ------------------------------------------------------------
    public String toString() {
	
	return "MessageForwarder(address=" + addr_ + ")";
    }
}
