/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.pubsub;

import java.io.OutputStream;
import java.io.PrintStream;
import java.lang.Runtime;
import java.lang.Process;

import casa.dodwan.message.Message;
import casa.dodwan.docware.Descriptor;
import casa.util.Processor;
import casa.dodwan.pubsub.PubSubService;

public class MessagePiper implements Processor<Descriptor> {

    private String cmd_;
    private PubSubService pubsub_;

    public MessagePiper(String cmd, PubSubService pubsub) {

	// System.out.println("Creating message saver for " + cmd);
	cmd_ = cmd;
	pubsub_ = pubsub;
    }
    
    // ------------------------------------------------------------
    public void process(Descriptor desc) throws Exception {

	// Extracting attributes from descriptor
	String key = desc.getDocumentId();

	// Retrieving payload from cache
	byte[] payload = pubsub_.getAsBytes(key);

	// Processing message
	System.out.println("Piping msg " + key + " to " + cmd_);

	String args = "";
	for (String name: desc.getNames()) 
	    args += " " + name + "=\"" + desc.getAttribute(name) + "\"";

	// Saving message payload in local file
	try {
	    Process process = Runtime.getRuntime().exec(cmd_ + " " + args);
	    PrintStream out = new PrintStream(process.getOutputStream());
	    out.write(payload);
	    out.flush();
	    out.close();
	}
	catch (Exception e) {
	    System.out.println("Failed to pipe to " + cmd_);
	}
    }
	

    // ------------------------------------------------------------
    public String toString() {
	
	return "MessagePiper(cmd=" + cmd_ + ")";
    }
}
