/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.pubsub;

import java.util.HashMap;
import java.util.Map;
import java.util.Date;
import java.io.*;

import casa.dodwan.docware.Descriptor;

public class ReceptionHistory {

    private Map<String,Long> history_ = 
	new HashMap<String,Long>();

    private PrintStream fout_ = null;

    public ReceptionHistory(File file) 
	throws Exception {

	File bakFile = new File(file.getAbsoluteFile() + ".bak");
	
	if (! file.exists()) {
	    fout_ =
		new PrintStream(new FileOutputStream(file));
	    return;
	}

	if (bakFile.exists()) {
	    bakFile.delete();
	}
		
	boolean renamed = file.renameTo(bakFile);
	if (!renamed) {
	    System.out.println("ReceptionHistory(); failed to rename "
			       + file.getAbsoluteFile() + " to "
			       + bakFile.getAbsoluteFile());
	    return;
	}

	fout_ =
	    new PrintStream(new FileOutputStream(file));
	BufferedReader fin  =
	    new BufferedReader(new FileReader(bakFile));
	
	long now = System.currentTimeMillis();
	String line = fin.readLine();
	while (line != null) {
	    String[] args = line.split(",");
	    String key = args[0];
	    long deadline = Long.parseLong(args[1]);

	    if (deadline > now) {
		history_.put(key, new Long(deadline));
		fout_.println(line);
	    }
	    line = fin.readLine();
	}

	fin.close();
	fout_.flush();
    }

    
    // ------------------------------------------------------------
    public void newMessageReceived(Descriptor desc) {

	String docId = desc.getDocumentId();
	Date deadline = desc.getDeadline();

	if (deadline == null) {
//  	    System.out.println("ReceptionHistory.newMessageReceived(): no deadline attribute in message");
	    return;
	}

	save(docId, deadline.getTime());
    }

    // ------------------------------------------------------------
    public boolean contains(String key) {

	return history_.containsKey(key);
    }

    // ------------------------------------------------------------
    public long getDeadline(String key) {

	return history_.get(key);
    }

    // ------------------------------------------------------------
    public void save(String key, long deadline) {

	if (history_.containsKey(key)) {
	    System.out.println("ReceptionHistory.save("
			       + key + ", " 
			       + deadline + "): key is already in history");
	    return;
	}

	history_.put(key, new Long(deadline));
	fout_.println(key + "," + deadline);
	fout_.flush();
    }

    // ------------------------------------------------------------
    public void purgeBefore(long threshold) {

	for (String key : history_.keySet()) {
	    long time = history_.get(key);
	    if (time < threshold)
		history_.remove(key);
	}
    }

    // ------------------------------------------------------------
    public void purgeAfter(long threshold) {

	for (String key : history_.keySet()) {
	    long time = history_.get(key);
	    if (time > threshold)
		history_.remove(key);
	}
    }
}
