/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.run;

import java.io.BufferedReader;
import java.io.PrintStream;
import java.io.File;
import java.net.InetSocketAddress;

import casa.dodwan.util.Console;
import casa.dodwan.util.ArgumentNotFoundException;
import casa.dodwan.util.ArgumentParsing;
import casa.dodwan.util.ArgumentParsingException;
import casa.dodwan.util.Command;
import casa.dodwan.config.DodwanEnvironment;
import casa.dodwan.util.SystemEnvironment;

public class DodwanConsole extends Console {
    
 
    // ============================================================
    // MAIN COMMANDS
    // ============================================================
    
    public Dodwan dodwan_;
    
    // ------------------------------------------------------------
    public DodwanConsole(Dodwan dodwan) {
	
	dodwan_ = dodwan;
	prompt_ = "% ";
	
	addSubConsole("t", "[t]ransmission", dodwan_.transmissionService.console());
	addSubConsole("ca", "[ca]che", dodwan_.cacheService.console());
	addSubConsole("g", "[g]ossiping", dodwan_.gossipingService.console());
	addSubConsole("ps", "[ps]", dodwan_.pubSubService.console());
	addSubConsole("q", "[q]", dodwan_.queueService.console());
    }
    
    // ============================================================
    // COMMAND PARSING
    // ============================================================
    
    // ------------------------------------------------------------
    @Override
	public void help(String offset, PrintStream out) {
	out
	    .println(offset
		     + "[su]spend / [resu]me\n"
		     + offset
		     + "[on]line / [off]line\n"
		     + offset
		     + "[n]ame\n"
		     + offset
		     + "[c]learLogFiles\n"
		     + offset
		     + "[u]ploadLogFiles [hostname] [port]\n"
		     + offset
		     + "[req]uestLog <-h host> <-p pattern> <-l hh:mm:ss>\n"
		     + offset
		     + "[rese]tLogger <fname>\n"
		     + offset
		     + "[l]og <string>\n"
		     + offset
		     + "[st]atus\n"
		     + offset
		     + "[shutdown]\n");

	subHelp(offset, out);  
    }



    // ------------------------------------------------------------
    @Override
	public void processCommand(String[] arguments, BufferedReader in,
				   PrintStream out) {
	
//  	if (arguments.length < 1) {
//  	    help("", out);
//  	    return;
//  	}
	
//  	// Prepare arguments for sub-console
//  	String[] args = new String[arguments.length - 1];
//  	for (int i = 1; i < arguments.length; i++){
//  	    args[i - 1] = arguments[i];
//  	}
//  	
//  	// Switch to sub-console
//  	if (arguments[0].startsWith("ca")) {
//  	    dodwan_.cacheService.console().processCommand(args, in, out);
//  	    return;
//  	}
//  	
//  	if (arguments[0].startsWith("t")) {
//  	    dodwan_.transmissionService.console().processCommand(args, in, out);
//  	    return;
//  	}
//  	
//  	if (arguments[0].startsWith("pub")) {
//  	    publish(arguments, out);
//  	    return;
//  	}
//  	
//  	if (arguments[0].startsWith("ps")) {
//  	    dodwan_.pubSubService.console().processCommand(args, in, out);
//  	    return;
//  	}
	
	// Process commands locally
	if (arguments[0].startsWith("on")) {
	    online(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("off")) {
	    offline(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("cl")) {
	    clearLogFiles(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("l")) {
	    log(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("n")) {
	    out.println(SystemEnvironment.host);
	    return;
	}
	
	if (arguments[0].startsWith("su")) {
	    suspend(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("req")) {
	    requestLog(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("resu")) {
	    resume(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("rese")) {
	    resetLogger(arguments, out);
	    return;
	}

	if (arguments[0].startsWith("st")) {
	    status(arguments, out);
	    return;
	}

//	if (arguments[0].startsWith("u")) {
//	    uploadLogFiles(arguments, out);
//	    return;
//	}

	if (arguments[0].startsWith("shutdown")) {
	    shutdown(arguments, out);
	    return;
	}

  	super.processCommand(arguments, in, out);

    }
    
    // ------------------------------------------------------------
    public void requestLog(String[] arguments, PrintStream out) {

	String host = arguments[1];
	String pattern = arguments[2];

        long lifetime = 300000;
        
        try {
            host = ArgumentParsing.searchString("-h", arguments);
            pattern = ArgumentParsing.searchString("-p", arguments);
            
            try {
                lifetime = ArgumentParsing.search_hh_mm_ss("-l", arguments);
            }
            catch (ArgumentNotFoundException e) {}
	}
	catch (ArgumentNotFoundException e) {
	    Command.missingArgument(e);
	    return;
	} 
	catch (ArgumentParsingException e) {
	    Command.parsingException(e);
	    return;
	}
	catch (Exception e) {
	    out.println(e);
	}

	dodwan_.logManagement.sendRequest(host, pattern, lifetime);
    }
    
    // ------------------------------------------------------------
    public void resume(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}
	
	dodwan_.resume();
//  	dodwan_.transmissionService.resume();
//  	dodwan_.cacheService.resume();
//  	dodwan_.pubSubService.resume();
    }
    
    // ------------------------------------------------------------
    public void suspend(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}

	dodwan_.suspend();
//  	dodwan_.pubSubService.suspend();
//  	dodwan_.cacheService.suspend();
//  	dodwan_.transmissionService.suspend();
    }

    // ------------------------------------------------------------
    public void online(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}
	
	dodwan_.online();
    }
    
    // ------------------------------------------------------------
    public void offline(String[] arguments, PrintStream out) {
	
	dodwan_.offline();
    }

    // ------------------------------------------------------------
    public void log(String[] arguments, PrintStream out) {
	
	if (arguments.length <= 1) {
	    help("", out);
	    return;
	}

	String strLog = "";
	for (int i=1; i<arguments.length; i++)
	    strLog += arguments[i] + " ";
	
	casa.util.Logger.log("dodwan", strLog);
    }
    
    // ------------------------------------------------------------
    public void status(String[] arguments, PrintStream out) {
	
	out.println("DoDWAN is "
		    + (dodwan_.isOnline()?"online":"offline"));
    }

    // ------------------------------------------------------------
    public void clearLogFiles(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}

	out.println("Clearing log files");
	File dir = new File(DodwanEnvironment.getInstance().directory, "log");
	casa.util.Logger.clear(dir);
    }

    // ------------------------------------------------------------
//    public void uploadLogFiles(String[] arguments, PrintStream out) {
//	
//	if (arguments.length != 3) {
//	    help("", out);
//	    return;
//	}
//
//	String host = arguments[1];
//	int port = Integer.parseInt(arguments[2]);
//	InetSocketAddress sockAddr = new InetSocketAddress(host, port);
//
//	out.println("Uploading log files to " + sockAddr);
//	File dir = new File(DodwanEnvironment.getInstance().directory, "log");
//	casa.util.Logger.upload(dir, sockAddr);
//	
//    }

    // ------------------------------------------------------------
    public void resetLogger(String[] arguments, PrintStream out) {
	
	if (arguments.length > 2) {
	    help("", out);
	    return;
	}

	out.println("Resetting logger");
	if (arguments.length == 1)
	    dodwan_.resetLogger();
	else
	    dodwan_.resetLogger(arguments[1]);
    }

    // ------------------------------------------------------------
    public void shutdown(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("", out);
	    return;
	}

	dodwan_.doNotify();
    }
	
}
