/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.run;

/**
 * Interface for a Dodwan plugin
 * An implementation of this interface should be named after the name of the plugin, and be in package "casa.dodwan.plugin"
 * The name of the plugin is in lower case with words separated with a dash (minus sign).
 * The corresponding implementation class name follows the Java usage with initial capitals and without dashes.
 * For example, for the plugin "dodwan-napi-ws", the class that implements the Plugin interface
 * is named "DodwanNapiWs" and its full name is "casa.dodwan.plugin.DodwanNapiWs".
 * 
 * Plugins are activated at startup, in dodwand: for each plugin specified (in order) 
 * in the dodwan.plugins property, the object is created and the method startPlugin() is called.
 */
public interface Plugin {
    
    void startPlugin(dodwand dodwand);
    void stopPlugin();
}
