/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Locale;


public class ActivityMonitor extends Thread {

    private long period_;
    private PrintStream ps_ = null;

    static final SimpleDateFormat dateFormat = 
	    new SimpleDateFormat("HH:mm:ss.SSS", Locale.ENGLISH);


    // ------------------------------------------------------------
    public ActivityMonitor(long period, 
			   File file)
	throws Exception {
	
	setDaemon(true);
	
   		period_ = period;
   	   	if (period <= 0) {
   	   			throw new Exception("Invalid activity period");
   	   	}
   	   	else {

    		try {
    			File dir = file.getParentFile();
    			if (dir != null)
    				dir.mkdirs();

    			ps_ = new PrintStream(new FileOutputStream(file, true));
    		}
    		catch (Exception e) {
    			System.out.println("ActivityMonitor(" + file.getAbsolutePath() 
    					+ "): failed to open file");
    		}
    	}
    }

    // ------------------------------------------------------------
    public void run() {
	
    		
    		long onTime = System.currentTimeMillis();
    		long lastCheck = onTime;
    		ps_.println("\n# On\t\tOff\t\tOn-Off (ms)\tOff-On (ms)");
    		ps_.print(dateFormat.format(onTime) + "\t");

    		while (true) {
    			long now = System.currentTimeMillis();
    			long elapsed = now - lastCheck;

    			if (elapsed > 4 * period_) {

    				long offTime = lastCheck;
    				long onOff = offTime - onTime;
    				long offOn = now - offTime;
    				onTime = now;

    				ps_.println(dateFormat.format(offTime) 
    						+ "\t" + onOff
    						+ "\t\t" + offOn);
    				ps_.print(dateFormat.format(onTime) + "\t");
    			}

    			lastCheck = now;

    			try {
    				sleep(period_);
    			}
    			catch (Exception e) {}
    		}
    	}
   
	
}
