/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

/**
 * A <code>Bufferizable</code> is a part of a message.
 */
public interface Bufferizable {

	/**
	 * Extracts a new value for this <code>Bufferizable</code> from the
	 * specified buffer, at the specified location.
	 * 
	 * @exception BufferAccessException
	 *                if a value for this <code>Bufferizable</code> cannot be
	 *                extracted from the buffer.
	 */
	public void fromBuffer(byte[] buffer, int offset, int length)
			throws BufferAccessException;

	/**
	 * Inserts the value of this <code>Bufferizable</code> in the specified
	 * buffer, at the specified location.
	 * 
	 * @return the number of bytes written to the buffer.
	 * 
	 * @exception BufferAccessException
	 *                if this <code>Bufferizable</code> cannot be inserted in
	 *                the buffer.
	 */
	public int toBuffer(byte[] buffer, int offset) throws BufferAccessException;
}
