/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/




package casa.dodwan.util;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Locale;

import casa.dodwan.cache.CacheService;
import casa.dodwan.gossip.GossipingService;
import casa.dodwan.config.DodwanEnvironment;
import casa.dodwan.message.Message;
import casa.dodwan.pubsub.PubSubService;
import casa.dodwan.queue.QueueService;
import casa.dodwan.sendreceive.SendReceiveService;
import casa.dodwan.transmission.TransmissionService;
import casa.dodwan.util.Console;
import casa.util.Logger;
import casa.dodwan.util.Service;
import casa.dodwan.util.SystemEnvironment;
import casa.dodwan.util.Environment;
import casa.dodwan.util.LogManagement;

/**
 * DoDWAN main service. This class should be instantiated only once, unless it
 * is used for simulation purposes.
 */
public class Domain {

    public Environment environment;

    public CacheService cacheService;
    
    public GossipingService gossipingService;
    
    public TransmissionService transmissionService;

    private Console console_ = null;

    // --------------------------------------------------------------------
    /**
     *  
     */
    public Domain(CacheService cacheService) throws Exception {

	// ------------------------------------------------------------
	// ENVIRONMENT MANAGEMENT
	environment = new Environment();

	// -------------------------------------------------
	// TRANSMISSION SERVICE
	transmissionService = new TransmissionService(environment);
	
	// -------------------------------------------------
	// GOSSIPING SERVICE
	gossipingService = 
	    new GossipingService(transmissionService,
				 cacheService.cache, environment);
	transmissionService.dispatcher.addSink(gossipingService);
	
	// --------------------------------------------------
	// ENABLING SERVICES
	transmissionService.enable();
	gossipingService.enable();
    }

    // ------------------------------------------------------------
    public Console console() {
	if (console_ == null)
	    console_ = new DomainConsole(this);
	return console_;
    }
   
    // ------------------------------------------------------------
    public void suspend() {

	gossipingService.suspend();
	transmissionService.suspend();
	environment.clearPeers();
    }
    
    // ------------------------------------------------------------
    public void resume() {

	transmissionService.resume();
	try {
	    Thread.sleep(200);
	}
	catch (Exception e) {}
	gossipingService.resume();
    }
    
}

// FG:
// 
// 1) qui fait suspend/resume ? (Dodwan ? Domain est un Service ?)
// 2) comment fonctionne la console ? une sous-console par domaine ?
// 3) création (et éventuellement suppression) de domaines ?
