/* *****************************************************************************
 * Copyright (c) 2005-2010 VALORIA Laboratory, 
 * Universite Europeenne de Bretagne, Universite de Bretagne-Sud, France
 * <http://www-valoria.univ-ubs/CASA/DoDWAN>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************
 * $Id: Log2DGS.java $
 * ****************************************************************************/

package casa.dodwan.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
    
/**
 * Parses a log file and detects all events pertaining to node
 * activity (i.e. online/offline events) as well as neighbor discovery
 * (i.e. add/remove neighbor). Produces a DGS file based on these
 * events.
 */
public class LogShifter extends LogParser {

    private long offset = 0;

    // ------------------------------------------------------------
    public LogShifter(InputStream in, OutputStream out) {
	super(in, out);
    }

    // ------------------------------------------------------------
    public void processEvent(Event event) {

	if (offset == 0)
	    offset = ref_time - event.time;

	event.time = event.time + offset;
	out.println(event);
    }

    // ------------------------------------------------------------
    public static void usage() {
        System.out.println("java LogShifter [ref_time] [-|logfile]");
        System.out.println("Reads a log file on stdin and produces the same file on stdout,");
	System.out.println("with timestamps shifted so the first event occurs at ref_time.");
        System.exit(0);
    }

   // ------------------------------------------------------------
    public static void main(String[] args) throws IOException {
        if ((args.length > 0 && args[0].equals("-h"))
	    || (args.length != 2)) {
            usage();
        }

	long ref_time = Long.parseLong(args[0]);
	
	InputStream is;
	if (args[1].equals("-"))
	    is = System.in;
	else
	    is = new FileInputStream(args[1]);
	
        LogShifter parser =
	    new LogShifter(is, System.out);
	parser.setRefTime(ref_time);
	
	parser.parse();
	
    }
}
