/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

import java.util.HashMap;

import casa.dodwan.message.Message;

public class Monitor implements Sink<Message> {

	private int allMessages_ = 0;

	private String name_;

	private Sink<Message> out_ = null;

	private HashMap<String, Integer> services_ = new HashMap<String, Integer>();

	private int stepMessages_ = 0;

	private HashMap<String, Integer> stepServices_ = new HashMap<String, Integer>();

	public Monitor(String name) {
		name_ = name;
	}

	public Monitor(String name, Sink<Message> sink) {
		name_ = name;
		out_ = sink;
	}

	public int getAllMessages() {
		return allMessages_;
	}

	public String getName() {
		return name_;
	}

	public int getServices(String serviceType) {
		if (services_.get(serviceType) != null) {
			System.out.println("Monitor : " + serviceType + " "
					+ services_.get(serviceType));
			return services_.get(serviceType);
		} else
			return 0;
	}

	public int getStepMessages() {
		return stepMessages_;
	}

	public int getStepServices(String serviceType) {
		if (stepServices_.get(serviceType) != null) {
			int value = stepServices_.get(serviceType);
			System.out.println("Monitor : " + serviceType + " " + value);
			stepServices_.put(serviceType, new Integer(0));
			stepMessages_ = 0;
			return value;
		} else
			return 0;
	}

	// TODO ecrire un méthode que rend du code HTML
	public String toHtml() {
		return name_ + "masc.util.Monitor : TODO ecrire la méthode";
	}

	public void write(Message message) throws Exception {

		String serviceType = message.getDescriptor().getAttribute("_stype");

		if (!services_.containsKey(serviceType)) {
			services_.put(serviceType, new Integer(1));
			stepServices_.put(serviceType, new Integer(1));
		} else {
			int value = services_.get(serviceType).intValue() + 1;
			services_.put(serviceType, value);
			int value2 = stepServices_.get(serviceType).intValue() + 1;
			stepServices_.put(serviceType, value2);
		}

		allMessages_++;
		stepMessages_++;

		if (out_ != null)
			out_.write(message);

	}

}
