/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/




package casa.dodwan.util;

import java.io.PrintStream;
import java.io.BufferedReader;

public class ServiceConsole extends Console {
    
    public Service service_;
    
    // ------------------------------------------------------------
    public ServiceConsole(Service service) {
	
	service_ = service;
	prompt_ = "% ";
    }
    
    // ------------------------------------------------------------
    public ServiceConsole(String prompt,
			  Service service) {
	
	service_ = service;
	prompt_ = prompt;
    }

    // ------------------------------------------------------------
    public void verbosity(String[] arguments, PrintStream out) {
	
	if (arguments.length < 2) {
	    help("  ", out);
	    return;
	}
	
	int level = -1;
	try {
	    level = Integer.parseInt(arguments[1]);
	} catch (Exception e) {
	    out.println("Could not parse verbosity level");
	}
	
	if (level > 0)
	    service_.verbosity.enable();
	else
	    service_.verbosity.disable();
	;
    }

    // ------------------------------------------------------------
    public void enable(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("  ", out);
	    return;
	}
	
	service_.enable();
    }
    
    // ------------------------------------------------------------
    public void disable(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("  ", out);
	    return;
	}
	
	service_.disable();
    }
    
    // ------------------------------------------------------------
    public void suspend(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("  ", out);
	    return;
	}
	
	service_.suspend();
    }
    
    // ------------------------------------------------------------
    public void resume(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("  ", out);
	    return;
	}
	
	service_.resume();
    }
    
    
    // ------------------------------------------------------------
    public void status(String[] arguments, PrintStream out) {
	
	if (arguments.length != 1) {
	    help("  ", out);
	    return;
	}
	
	System.out.println("Enabled: " + service_.isEnabled());
	System.out.println("Suspended: " + service_.isSuspended());
    }
    
    // ------------------------------------------------------------
    public void help(String offset, PrintStream out) {
	out.println(offset + "[e]nable / [d]isable\n" 
		    + offset + "[sus]pend / [res]ume\n" 
		    + offset + "[st]atus\n"
		    + offset + "[v]erbosity <level>\n");
    }

    // ------------------------------------------------------------
    public void processCommand(String[] arguments, BufferedReader reader,
			       PrintStream out) {
	
	if (arguments.length < 1) {
	    help("  ", out);
	    return;
	}
	
	if (arguments[0].startsWith("d")) {
	    disable(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("e")) {
	    enable(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("h")) {
	    out.println("Command Set\n");
	    help("  ", out);
	    return;
	}
	
	if (arguments[0].startsWith("res")) {
	    resume(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("st")) {
	    status(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("sus")) {
	    suspend(arguments, out);
	    return;
	}
	
	if (arguments[0].startsWith("v")) {
	    verbosity(arguments, out);
	    return;
	}
	
	out.println("Unknown command");
	help("  ", out);
    }
    
}   
