/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

public class TimeParser {

    // ------------------------------------------------------------
    public static long from_dd_hh_mm(String str) {

	str = str.replace(" ", "");

	long result = 0;

	if (str.contains(":")) {

	    // Parsing a dd:hh:mm formatted string
	    //
	    // Examples:
	    // "03:12:30"
	    // "3:12:30"
	    // "12:30"
	    // "30"
	    String[] ddhhmm = str.split(":");
	    
//  	    System.out.print("ddhhmm=");
//  	    for (String vv: ddhhmm) 
//  		System.out.print(vv + " ");
//  	    System.out.println();

	    long dd = 0;
	    long hh = 0;
	    long mm = 0;
	    try {
		if (ddhhmm.length > 0)
		    mm = Long.parseLong(ddhhmm[ddhhmm.length-1]);
		if (ddhhmm.length > 1)
		    hh = Long.parseLong(ddhhmm[ddhhmm.length-2]);
		if (ddhhmm.length > 2)
		    dd = Long.parseLong(ddhhmm[ddhhmm.length-3]);
	    }
	    catch (Exception e) {
		result = -1;
	    }
	    
	    result = mm + 60 * (hh + 24 * dd);
	}

	else {

	    // Parsing a free-form string with quantifiers
	    //
	    // Examples:
	    // " 3 days 12 hours 30 min"
	    // "3 d 12 h 30 m"
	    // "3 j 12 h 30 min"
	    // "3j12h30m"
	    // "12h30m"
	    // "30m"
	    String[] quantifiers = str.split("[\\d]+");
	    String[] numbers = str.split("[\\D]+");

//  	    System.out.println("str='" + str + "'");
//  	    System.out.println("nbn=" + numbers.length
//  			       + ", nbq=" + quantifiers.length);
//  	    for (String n: numbers)
//  		System.out.print(n + " / ");
//  	    System.out.println();
//  	    for (String q: quantifiers)
//  		System.out.print((q.equals("")?"-":q.substring(0, 1))
//  				 + " / ");
//  	    System.out.println();

	    if (numbers.length == 0)
		return -1;

	    result = 0;
	    try {
		for (int i=0; i<numbers.length; i++) {
		    long val = Long.parseLong(numbers[i]);
		    long mult = 1;
		    if (i+1<quantifiers.length) {
			String quant = quantifiers[i+1].toUpperCase().substring(0, 1);
			if (quant.equals("D") || quant.equals("J"))
			    mult = 1440;
			else if (quant.equals("H"))
			    mult = 60;
			else if (quant.equals("M"))
			    mult = 1;
			else throw new Exception();
		    }
		    result += val * mult;
		}
	    } catch (Exception e) {
		result = -1;
	    }
	}

	return result;
    }
    
    // ------------------------------------------------------------
    public static String to_dd_hh_mm(long time) {
	
	long dd, hh, mm;
	mm  = time;
	dd  = mm / 1440;
	mm -= dd * 1440;
	hh  = mm / 60;
	mm -= hh * 60;

	return (dd < 10 ? "0" : "") + dd + ":" 
	    + (hh < 10 ? "0" : "") + hh + ":" 
	    + (mm < 10 ? "0" : "") + mm;
    }
    
    // ------------------------------------------------------------
    public static void main(String[] args) 
	throws Exception {

	java.io.BufferedReader in = 
	    new java.io.BufferedReader(new java.io.InputStreamReader(System.in));

	String line = "";
	String field = null;
	do {
	    line = in.readLine();
	    System.out.println("Parsing '" + line + "'");

	    long val = -1;
	    int beg = line.lastIndexOf("[");
	    int end = line.lastIndexOf("]");
	    System.out.println("beg=" + beg + ", end=" + end);
	    if ((beg > -1) && (end > beg)) {
		field = line.substring(beg + 1, end);
		System.out.println("field='" + field + "'");

		val = from_dd_hh_mm(field);
	    }

	    if (val < 1)
		System.out.println("Failed");
	    else 
		System.out.println("Result: '" 
				   + line.replace(field, to_dd_hh_mm(val))
				   + "'");
	}
	while (line != null);
    }
}
