/* *****************************************************************************
 * Copyright (c) 2005-2010 VALORIA Laboratory, 
 * Universite Europeenne de Bretagne, Universite de Bretagne-Sud, France
 * <http://www-valoria.univ-ubs/CASA/DoDWAN>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************
 * $Id: Log2DGS.java $
 * ****************************************************************************/

package casa.dodwan.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
    
/**
 * Parses a log file and converts all timestamps to EPOCHms format,
 * using the specified reference time. Produces on stdout a log file
 * with relative timestamps.
 */
public class toEPOCHms extends LogParser {

    // ------------------------------------------------------------
    public toEPOCHms(InputStream in, OutputStream out) {
	super(in, out);
    }

    // ------------------------------------------------------------
    public void processEvent(Event event) {

	// event.time = event.time - ref_time;
	out.println(event.time
		    + " " + event.name
		    + " " + event.tag + ": " + event.message);
    }

    // ------------------------------------------------------------
    public static void usage() {
        System.out.println("java toEPOCHms [ref_time] [-|logfile]");
        System.out.println("Parses a log file and converts all timestamps to EPOCHms format,");
	System.out.println("using the specified reference time. Produces on stdout a log file");
	System.out.println("with relative timestamps.");
        System.exit(0);
    }

   // ------------------------------------------------------------
    public static void main(String[] args) throws IOException {
        if ((args.length > 0 && args[0].equals("-h"))
	    || (args.length != 2)) {
            usage();
        }

	long ref_time = Long.parseLong(args[0]);
	
	InputStream is;
	if (args[1].equals("-"))
	    is = System.in;
	else
	    is = new FileInputStream(args[1]);

	System.out.println("# ref_time=" + ref_time);
        toEPOCHms parser =
	    new toEPOCHms(is, System.out);
	parser.setRefTime(ref_time);
	
	parser.parse();
	
    }
}
